#include <vpi_user.h>

#include <cv.h>
#include <highgui.h>

#define MAX_DSPY 8

static IplImage *img_sink;
static unsigned int row_sink, col_sink;

static int cv_init_display_calltf(char*c){
  vpiHandle sys_tf_ref, arg_iter;
  vpiHandle rows_h, cols_h;
  int rows, cols;
  s_vpi_value value;

  sys_tf_ref = vpi_handle(vpiSysTfCall, NULL);
  arg_iter = vpi_iterate(vpiArgument, sys_tf_ref);
  
  value.format = vpiIntVal;

  cols_h = vpi_scan(arg_iter);
  vpi_get_value(cols_h, &value);
  cols = value.value.integer;

  rows_h = vpi_scan(arg_iter);
  vpi_get_value(rows_h, &value);
  rows = value.value.integer;

  vpi_printf("Initialize display %dx%d\n", cols, rows);

  img_sink = cvCreateImage( cvSize(cols, rows), IPL_DEPTH_8U, 1);
  cvNamedWindow("Image",0);

  row_sink = col_sink = 0;
}


static int eol;
static int eof;

static int cv_clk_display_calltf(char*c){
  IplImage *img;
  vpiHandle sys_tf_ref, arg_iter;
  vpiHandle vact_h, hact_h, din_h;
  s_vpi_value value;

  sys_tf_ref = vpi_handle(vpiSysTfCall, NULL);
  arg_iter = vpi_iterate(vpiArgument, sys_tf_ref);
  
  value.format = vpiIntVal;
  
  img = img_sink;
  
  if(img == NULL) {
    vpi_printf("Error: display not initialized\n");
    return -1;
  }

  vact_h = vpi_scan(arg_iter);
  vpi_get_value(vact_h, &value);
  
  if(value.value.integer == 0) {
    if(eof) {
      eof = 0;
      vpi_printf("Frame Complete\n");
      cvShowImage("Image",img);
      cvWaitKey(0);
    }
    row_sink = col_sink = 0;
    eol = 0;
    return 0;
  }

  hact_h = vpi_scan(arg_iter);
  vpi_get_value(hact_h, &value);
  if(value.value.integer == 0) {
    if(eol == 1){
      row_sink++;
    }
    eol = 0;
    col_sink = 0;
    return 0;
  }


  din_h = vpi_scan(arg_iter);
  vpi_get_value(din_h, &value);
  
  /* do it */
  img->imageData[ (row_sink * img->widthStep) + col_sink ] = (unsigned char)value.value.integer;
  
  col_sink++;
  eol = 1;
  eof = 1;

  return 0;
}

void display_register(void)
{
  s_vpi_systf_data tf_data;

  tf_data.type      = vpiSysTask;
  tf_data.tfname    = "$cv_init_display";
  tf_data.calltf    = cv_init_display_calltf;
  tf_data.compiletf = 0;
  tf_data.sizetf    = 0;
  tf_data.user_data = "$cv_init_display";
  vpi_register_systf(&tf_data);

  tf_data.type      = vpiSysTask;
  tf_data.tfname    = "$cv_clk_display";
  tf_data.calltf    = cv_clk_display_calltf;
  tf_data.compiletf = 0;
  tf_data.sizetf    = 0;
  tf_data.user_data = "$cv_clk_display";
  vpi_register_systf(&tf_data);

}

void (*vlog_startup_routines[])() = {
  display_register,
  0
};
