`timescale 1ns/1ps

module testbench();
   
   parameter CLK0_PER = 8.0;	//125MHz
   parameter CLK1_PER = 25;	//40MHz
   parameter CLK3_PER = 50;	//20MHz

   // Inputs
   wire [7:0] PXD;
   wire       BPF;
   wire       HACT;
   wire       VACT;

   reg 	      TTRIG;
   reg 	      CLK3;
   reg 	      CLK2;
   reg 	      CLK1;
   reg 	      CLK0;

   // Outputs
   wire       DCLK;
   reg 	      MRST;
   reg 	      ARO;
   reg 	      ARST;
   wire       CNVSYNC;
   wire       CNVCLK;
   wire       XRST;
   wire       AUXCLK;
   wire [14:0] SDA;
   wire        SDCLK, SDNCLK;

   // Bidirs
   wire        SCL0;
   wire        SCL1;
   wire        SDA0;
   wire        SDA1;
   wire        EXPS;
   wire        TRIG;
   wire [15:0] SDD;
   wire [31:0] D;
   wire        LDQS;
   wire        UDQS;

   /* Setup Simulation parameters */
   initial begin 
      // uncomment the following for .vcd ouput files
      // $dumpfile("testbench.vcd");
      // $dumpvars(1, testbench);
   end
   
   /* Simulate for a fixed number of frames */
   parameter last_frame = 4;
   
   integer   frame_count;
   initial frame_count = 0;
   always @ (posedge VACT) begin
      if(frame_count == last_frame)
	$finish;
      frame_count <= frame_count + 1;
      $display("new frame %d", frame_count);
   end
   
   /* Initialize variables */
   initial begin
      CLK0 = 0;
      CLK1 = 0;
      CLK3 = 0;
   
      MRST = 1;
      ARO = 1;
   end

   /* Set clocks */
   always #(CLK0_PER/2)	CLK0 =	~CLK0;
   always #(CLK1_PER/2)	CLK1 =	~CLK1;
   always #(CLK3_PER/2)	CLK3 =	~CLK3;

   assign     DCLK = CLK1;

   /*******************************************************************/
   // Simulation Modules
   
   // Image Sensor; define the image file:
   defparam   i_ccd.source = "image.jpg";
   sensor i_ccd(.MCLK(DCLK),    // Master clock
		.MRST(MRST),    // Master Reset - active low
		.ARO(ARO),      // Array read Out.
		.ARST(ARST),    // Array Reset. Active low
		.OE(1'b0),      // output enable active low
		.SCL(SCL0),     // I2C clock
		.SDA(SDA0),     // I2C data
		.OFST(1'b1),    // I2C address ofset by 2: for simulation 0 - still mode, 1 - video mode.
		.D(PXD[7:0]),   // data output
		.DCLK(),        // Data output clock
		.BPF(BPF),      // Black Pixel Flag
		.HACT(HACT),    // Horizontal Active
		.VACT(VACT)     // Vertical Active
		);
   
   
   initial begin 
      #10
	$cv_init_display(i_ccd.ncols, i_ccd.nrows);
   end
   
   /********** DISPLAY *************/
   
   always @(posedge DCLK) 
     $cv_clk_display(VACT, HACT, PXD);

   
endmodule // testbench
